function M = hansenm(ell,m,u,theta,phi)
%HANSENM Hansen solution M of the spherical vectorial wave equation 
%   M = HANSENM(ELL,M,R,U,THETA,PHI,K,ORDER) computes Hansen solution M
%       of the vectorial wave equation for degree ell and order m 
%       (-ell <= m <= ell) at point (r,theta,phi) in spherical coordinates
%       k is the wave number involved in the vectorial wave equation 

% VARIABLES IN :
% ell : polar mode index
% m : azimuthal mode index
% u : radial variation of the potential
% theta : polar coordinate of the points where the EM field is to be computed
% phi : azimuthal coordinate of the points where the EM field is to be computed
%
% VARIABLE OUT :
% M : Hansen function M at point (theta,phi) for degree ell and order m 
%
% Author: Stephane BALAC, IRMAR, CNRS, University of Rennes, France
% contact: stephane.balac@univ-rennes1.fr
% Last modification: 17 june 2019
%

% Computation of the constant normalization for the Spherical Surface Harmonics 
% when computing Associated Legendre function with Schmidt normalization
if m~=0
    C=(-1)^m*sqrt((2*ell+1)/(8*pi));
else
    C=sqrt((2*ell+1)/(4*pi));
end

if theta~=0
    M(1,:) = 0;
    M(2,:) = (1i*m*C./sin(theta)).*u.*slegend(ell,m,cos(theta)).*exp(1i*m*phi);
    M(3,:) = C*sin(theta).*u.*dslegend(ell,m,cos(theta)).*exp(1i*m*phi);
else
    M(1,:) = 0;
    M(2,:) = 0;
    M(3,:) = C*sin(theta).*u.*dslegend(ell,m,cos(theta)).*exp(1i*m*phi);
end
end
