function N = hansenn(ell,m,k,Nopt,u,du,r,theta,phi)
%HANSENN Hansen solution N of the spherical vectorial wave equation 
%   N = HANSENN(ELL,M,R,THETA,PHI,K,ORDER) computes Hansen solution N
%       of the vectorial wave equation for degree ELL and order M 
%       (-ELL <= M <= ELL) at point (r,theta,phi) in spherical coordinates

% VARIABLES IN :
% ell : polar mode index
% m : azimuthal mode index
% k : resonance wavenumber in mu-m^(-1)
% Nopt : optical index at grid nodes
% u : radial variation of the potential
% du : derivative of the potential u in the radial direction
% r : radial coordinate of the points where the EM field is to be computed
% theta : polar coordinate of the points where the EM field is to be computed
% phi : azimuthal coordinate of the points where the EM field is to be computed
%
% VARIABLE OUT :
% N : Hansen function N at point (r,theta,phi) for degree ell and order m 
%
% Author: Stephane BALAC, IRMAR, CNRS, University of Rennes, France
% contact: stephane.balac@univ-rennes1.fr
% Last modification: 17 june 2019


% Computation of the constant normalization for the Spherical Surface Harmonics 
% when computing Associated Legendre function with Schmidt normalizationif m~=0
if m~=0
    C=(-1)^m*sqrt((2*ell+1)/(8*pi));
else
    C=sqrt((2*ell+1)/(4*pi));
end

if theta~=0 && r~=0
    N(1,:) = (C*ell*(ell+1)./(k*r.*Nopt)).*u.*slegend(ell,m,cos(theta)).*exp(1i*m*phi);
    N(2,:) = -(C./(k*r.*Nopt)).*(u+r.*du).*sin(theta).*dslegend(ell,m,cos(theta)).*exp(1i*m*phi);
    N(3,:) = (1i*m*C./(k*r.*Nopt*sin(theta))).*(u+r.*du)*slegend(ell,m,cos(theta)).*exp(1i*m*phi);
elseif theta==0 && r~=0
    N(1,:) = (C*ell*(ell+1)./(k*r.*Nopt)).*u.*slegend(ell,m,cos(theta)).*exp(1i*m*phi);
    N(2,:) = 0;
    N(3,:) = 0;
else
    N(1,:) = 0;
    N(2,:) = 0;
    N(3,:) = 0;
end

end

