function [A,C]=matrice(Qi,Q,q,p,d,R,r,dr,N,pmld,pmldt)
% MATRICE : computes the matrix of the Finite Difference approximation of the
% resonance problem

% VARIABLES IN :
%
% Qi : [int] index of the grid node located at the cavity boundary
% Q : [int] index of the grid node located at the PML outer side
% q : [int] parameter defining the regularity of the PML (0,1,2)
% p : [int] parameter used to distinguish between TE and TM modes
% d : [int] parameter used to distinguish between the resonance problem 
%            for a sphere and for a disk
% R : [double] radius of the disk or sphere
% r : [array of double] position of the grid nodes
% dr : [double] step size of the discretization
% N : [handle function] optical index 
% pmld : [handle function] function d involved in the definition of the PML
% dpmldt : [handle function] function d tilda involved in the definition of the PML
%
% VARIABLES OUT:
%
% A : [array of complex] the matrix of the Finite Difference approximation of the
% resonance problem 
%
% Author: Stephane BALAC, IRMAR, CNRS, University of Rennes, France
% contact: stephane.balac@univ-rennes1.fr
% Last modification: 15 july 2020
% Program distributed under the CeCILL-C Public License


% quantities involved in the matrix coefficients definition
if d==1 % disk
    c1=@(r) (N(r).^(p-1).*r).*(r<=R) + (r.*pmldt(r)./pmld(r)).*(r>R);
    c2=@(r) (q^2.*N(r).^(p-1)./r).*(r<=R)+ (q^2.*pmld(r)./(r.*pmldt(r))).*(r>R);
    c3=@(r) (N(r).^(p+1).*r).*(r<=R)+(r.*pmld(r).*pmldt(r)).*(r>R);
else
    c1=@(r) (N(r).^(p-1)).*(r<=R) + (1./pmld(r)).*(r>R);
    c2=@(r) (q^2.*N(r).^(p-1)./r.^2).*(r<=R)+ (q^2.*pmld(r)./((r.*pmldt(r)).^2)).*(r>R);
    c3=@(r) (N(r).^(p+1)).*(r<=R)+pmld(r).*(r>R);
end
%
% Construction of the matrices of the system
%
Qe=Q-1;
% Matrix A
% First term of the differential equ.
B1=-c1(r-0.5*dr).*ones(1,Qe);
B1=[B1(2:length(B1)),0];
B2=(c1(r+0.5*dr)+c1(r-0.5*dr)).*ones(1,Qe);
B3=-c1(r+0.5*dr).*ones(1,Qe);
B3=[0,B3(1:length(B3)-1)];
B=[B1;B2;B3];
B=transpose(B)./dr^2;
A1=spdiags(B,[-1,0,1],Qe,Qe);
% second term of the differential equ.
B2=c2(r).*ones(1,Qe);
B2=transpose(B2);
A2=spdiags(B2,0,Qe,Qe);
% The full matrix
A=A1+A2;
%
B2=c3(r).*ones(1,Qe);
B2=transpose(B2);
C=spdiags(B2,0,Qe,Qe);
% 
% Taking into account of the interface condition at dielectric boundary / PML entry
%
if d==1 % case of a disk
    C(Qi,Qi)=0.5*R*(1+N(R).^(p+1));
    A(Qi,Qi)=(c1(R+0.5*dr)+c1(R-0.5*dr))/dr^2+ ...
              0.5*(q^2/R)*(1+ N(R)^(p-1));
else % case of a sphere
     C(Qi,Qi)=0.5*(1+N(R)^(p-1)*R);
     A(Qi,Qi)=(c1(R+0.5*dr)+c1(R-0.5*dr))/dr^2+ ...
         0.5*(q^2/R^2)*(1+ N(R)^(p-1)) ;
end
