function [E,H]=modefield(ell,m,r,theta,phi,u,du,tetm,k,N)
% MODEFIELD : computes the electric field and magnetic field amplitude of a
% given mode defined by its mode indexes (ell,m) and a given resonance
% wavenumber k for a micro-sphere 

% VARIABLES IN :
% ell : polar mode index
% m : azimuthal mode index
% r : radial coordinate of the points where the EM field is to be computed
% theta : polar coordinate of the points where the EM field is to be computed
% phi : azimuthal coordinate of the points where the EM field is to be computed
% u : radial variation of the potential
% du : derivative of the potential u in the radial direction
% tetm : selector for TE or TM modes
% k : resonance vavenumber in mu-m^(-1)
% N : optical index at grid nodes
%
% VARIABLES OUT:
% E : electric field amplitude at (r,theta,phi)
% H : magnetic field amplitude at (r,theta,phi)
%
% Author: Stephane BALAC, IRMAR, CNRS, University of Rennes, France
% contact: stephane.balac@univ-rennes1.fr
% Last modification: 15 August 2020
% Program distributed under the CeCILL-C Public License

Z0 = 376.730313667; %  impedance of free space in Ohm
cste=k*Z0;

% Computation of the electromagnetic field
if strcmp(tetm,'TE') % TE mode
    for i=1:length(r)
        Mhans=hansenm(ell,m,u(i),theta,phi); % Computation of Hansen function M
        E(i)=norm(real(Mhans));
        % Computation of Hansen function N
        Nhans=-(1i*k*N(i)/cste)*hansenn(ell,m,k,N(i),u(i),du(i),r(i),theta,phi); 
        H(i)=norm(real(Nhans));
    end
else % TM mode
    for i=1:length(r)
        Mhans=-(1i/cste)*hansenm(ell,m,u(i),theta,phi); % Computation of Hansen function M
        H(i)=norm(real(Mhans));
        % Computation of Hansen function N
        Nhans=(1./(k*N(i)))*hansenn(ell,m,k,N(i),u(i),du(i),r(i),theta,phi);
        E(i)=norm(real(Nhans));
    end
end
