% RSPHERE : Display the electric field and magnetic field amplitude for a micro-sphere cavity
% An internal program called by WGRIN.
%
% Author: Stephane BALAC, IRMAR, CNRS, University of Rennes, France
% contact: stephane.balac@univ-rennes1.fr
% Last modification: 17 July 2020

prompt='[?] Value of the external boundary untill where the mode is displayed = ';
Ro = input(prompt) ;
while Ro<R
    disp(['[!] Enter a value larger than R = ',num2str(R)]);
    Ro = input(prompt) ;
end
Qo=round(Ro/dr);   % number of the node located at R2
ri=Rmin+[0:Qi]*dr; % nodes inside the cavity
ro=[Qi+1:Qo]*dr;   % nodes outside the cavity
newr=[ri,ro];
%----------------------------------------------
% number of points used in the radial and angular direction for
% image display purposes
% These values can be increased for a high image quality
% but memory requirement will be higher
if expert_mode==0
    nbpa=100; % the image is sampled with nbpa points in the 2 angular direction
    nbpr=500; % the image is sampled with nbpr points in the radial direction
else
    disp('    Image display quality')
    nbpa=input('[?] Number of sampling points in the 2 angular directions = ');
    if isempty(nbpa), nbpa=100; end
    nbpr=input('[?] Number of sampling points in the radial direction = ');
    if isempty(nbpr), nbpr=100; end
end

%----------------------------------------------
cntdism=1;
while cntdism==1
    prompt='[?] Azimuthal mode number m = ';
    m = input(prompt) ;
    while m>ell
        disp(['[!] Enter a value lower or equal to ell = ',num2str(ell)]);
        m = input(prompt) ;
    end
    tag=input('Name of the figures backup files (if empty, no backup) : ','s');
    fontsize=12;
    if expert_mode==1
        phi=input('[?] Azimuthal angle [radian] : phi = ');
        if isempty(phi), phi=0; end
        theta=input('[?] Polar angle [radian] : theta = ');
        if isempty(theta), theta=pi/2; end
    else
        phi=0;
        theta=pi/2;
    end
    Rmin=0;
    Rmax=Ro;
    %
    angle=[0:pi/nbpa:2*pi];
    xcp=R*cos(angle);
    ycp=R*sin(angle);
    angle=[-pi/2:pi/nbpa:pi/2];
    xct=R*cos(angle);
    yct=R*sin(angle);
    % Must be smaller than the number of discretization nodes
    if nbpr>Qo, nbpr=Qo; end
    npasRo=round(Qo/nbpr);
    pasRo=R0/nbpr;
    pRo=newr(1:npasRo:Qo);
    
    cntdisj=1;
    while cntdisj==1
        j=input('[?] Index of the computed modes to be displayed = ');
        figure('units','normalized','outerposition',[0 0 1 1]) % full screen UI window
        kj=k(j);
        lambdaj=2*pi/kj;
        Sj=transpose(V(:,j)); % current eigenvector (Debye potential)
        Uj=Sj./(kj*r); % potential involved in the expression of the EM field
        Uj=Uj./max(abs(Uj)); % normalisation
        U1=Uj(1:Qi);
        iU1=imag(U1);
        rU1=real(U1);
        alpha=0.5*atan(0.5*(sum(rU1.^2)-sum(iU1.^2))/sum(rU1.*iU1) );
        U1=U1*exp(1i*alpha); % normalisation
        U2=U1(Qi)*sqrt(ro/R).*besselh(ell+0.5,1,kj*ro)/besselh(ell+0.5,1,kj*R);
        U=[0,U1,U2];
        U=U./max(abs(U)); % normalisation
        % Approximation of the derivative of u by a second order finite difference formula
        dim=length(U);
        dU=(U(3:dim)-U(1:dim-2))/(2*dr);
        dU=[ (U(2)-U(1))/dr, dU, (U(dim)-U(dim-1))/dr];
        
        
        %=========================================================================
        % Plot the radial variation of the electromagnetic field
        %=========================================================================
        
        Vj=transpose(U(1:npasRo:Qo));
        dVj=transpose(dU(1:npasRo:Qo));
        
        [E,H]=modefield(ell,m,pRo,theta,phi,Vj,dVj,tetm,kj,N(pRo));
        E=E./max(E);
        H=H./max(H);
        subplot(2,3,1)
        [vR,iR]=max(pRo>R); % we must take into account the discontinuity of E across the interface
        if vR==1 && strcmp(tetm,'TM')  % E has a jump at the interface
            iR=iR-1;
            plot(pRo(1:iR),E(1:iR),'-b','LineWidth',3)
            hold on
            plot(pRo(1+iR:length(pRo)),E(1+iR:length(pRo)),'-b','LineWidth',3)
        else
            plot(pRo,E,'-b','LineWidth',3)
        end
        grid
        hold on
        plot([R,R],[0,1],'-.k','LineWidth',2); % position of the cavity boundary
        ylim([0,1])
        hold off
        text(0.5*(R+Ro),1/2,'Vacuum')
        text(0.75*R,1/2,'Disk')
        title(['Electric field amplitude for \lambda = ',num2str(real(lambdaj)),' \mu-m'])
        xlabel('Radial position in \mu-m')
        ylabel('Norm in arb. unit')
        subplot(2,3,4)
        plot(pRo,H,'-r','LineWidth',3)
        hold on
        plot([R,R],[0,1],'-.k','LineWidth',2); % position of the cavity boundary
        ylim([0,1])
        hold off
        text(0.5*(R+Ro),1/2,'Vacuum')
        text(0.75*R,1/2,'Disk')
        grid
        title(['Magnetic field amplitude for \lambda = ',num2str(real(lambdaj)),' \mu-m'])
        xlabel('Radial position in \mu-m')
        ylabel('Norm in arb. unit')
        
        %=========================================================================
        % Plot the electromagnetic field in the Azimuthal plane
        %=========================================================================
        
        Tmin=0; Tmax=pi;
        pasth=(Tmax-Tmin)/nbpa;
        [tht,rt] = meshgrid(Tmin:pasth:Tmax,pRo);
        [X,Y] = pol2cart(tht,rt);
        Vj=transpose(U(1:npasRo:Qo));
        dVj=transpose(dU(1:npasRo:Qo));
        
        E=zeros(size(rt));
        H=zeros(size(rt));
        
        for jj=1:size(tht,2)
            [Ei,Hi]=modefield(ell,m,rt(:,1),tht(1,jj),phi,Vj,dVj,tetm,kj,N(pRo));
            E(:,jj)=Ei;
            H(:,jj)=Hi;
        end
        E=E./max(max(E));
        H=H./max(max(H));
        %
        % Plot the electric field
        %
        colormap(jet)
        subplot(2,3,2)
        [VOID,h]=contourf(Y,X,E);
        h.LineStyle='none';
        colorbar
        axis equal
        xlabel(' \mu-m ','FontSize',fontsize);
        ylabel(' \mu-m ','FontSize',fontsize);
        title(['ell=',num2str(ell),', m=',...
            num2str(m),', \phi=',num2str(phi)],'FontSize',fontsize);
        hold on
        plot(xct,yct,'.-k');
        %
        % Plot the magnetic field
        %
        subplot(2,3,5)
        [VOID,h]=contourf(Y,X,H);
        h.LineStyle='none';
        colorbar
        axis equal
        xlabel(' \mu-m ','FontSize',fontsize);
        ylabel(' \mu-m  ','FontSize',fontsize);
        title(['ell=',num2str(ell),', m=',...
            num2str(m),', \phi=',num2str(phi)],'FontSize',fontsize);
        hold on
        plot(xct,yct,'.-k');
        
        %=========================================================================
        % Plot the electromagnetic field in the polar plane
        %=========================================================================
        
        Pmin=0; Pmax=2*pi;
        pasph=(Pmax-Pmin)/(2*nbpa);
        % generate meshgrid for the plots
        [pht,rt] = meshgrid(Pmin:pasph:Pmax,pRo);
        [X,Y] = pol2cart(pht,rt);
        
        E=zeros(size(rt));
        H=zeros(size(rt));
        for jj=1:size(pht,2)
            [Ei,Hi]=modefield(ell,m,rt(:,1),theta,pht(1,jj),Vj,dVj,tetm,kj,N(pRo));
            E(:,jj)=Ei;
            H(:,jj)=Hi;
        end
        E=E./max(max(E));
        H=H./max(max(H));
        %
        % Plot the electric field
        %
        subplot(2,3,3)
        [VOID,h]=contourf(X,Y,E);
        h.LineStyle='none';
        colorbar
        axis equal
        title([' \theta=',num2str(theta)],'FontSize',fontsize);
        xlabel(' \mu-m ','FontSize',fontsize);
        ylabel(' \mu-m  ','FontSize',fontsize);
        hold on
        plot(xcp,ycp,'.-k');
        %
        % Plot the magnetic field
        %
        subplot(2,3,6)
        [VOID,h]=contourf(X,Y,H);
        h.LineStyle='none';
        colorbar
        axis equal
        title(['\theta=',num2str(theta)],'FontSize',fontsize);
        xlabel(' \mu-m ','FontSize',fontsize);
        ylabel(' \mu-m  ','FontSize',fontsize);
        hold on
        plot(xcp,ycp,'.-k');
        if isempty(tag)==0, print([tag,num2str(j)],'-dpng'); end
        cntdisj=input('[?] Display a mode for an other radial mode index n (y/n) ? ans =  : ','s');
        if cntdisj=='y', cntdisj=1; else, cntdisj=0; end
    end
    cntdism=input('[?] Display a mode for an other azimuthal mode index m (y/n) ? ans =  : ','s');
    if cntdism=='y', cntdism=1; else, cntdism=0; end
end
